<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Cus00800 extends \Zeedhi\Framework\Controller\Report {
    
    protected $novoCodigo;
    protected $procedureConsumoPlanejado;
    protected $consplan;
    protected $servicoPorGrupo;
    protected $qr2Strategy;

    public function __construct(ReportService $reportService, $novoCodigo, $consplan, $servicoPorGrupo, $qr2Strategy, $procedureConsumoPlanejado) {
        $this->novoCodigo                = $novoCodigo;
        $this->consplan                  = $consplan;
        $this->servicoPorGrupo           = $servicoPorGrupo;
        $this->qr2Strategy               = $qr2Strategy;
        $this->procedureConsumoPlanejado = $procedureConsumoPlanejado;
        $this->reportService = $reportService;
    }

    protected $reportMapping = array (
        'cus00805' => 
            array (
                'strategy' => 'QR2',
                'parameterMapping' => 
                array (
                    'CDFILIAL'      => 'P_CDFILIAL',
                    'NRCONSPLAN'    => 'P_NRCONSPLAN',
                    'LISTASERV'     => 'P_CDSERVFILI',
                    'NIVEL'         => 'P_NIVEL',
                    'ORDENA'        => 'P_ORDENA', 
                    'DTINICIAL'     => 'P_DTINICIAL',
                    'DTFINAL'       => 'P_DTFINAL',
                    'DTPERIINIC'    => 'P_DTPERIINIC',
                    'DTPERIFINA'    => 'P_DTPERIFINA',
                    'CDPRODINI'     => 'P_CDPRODINIC',
                    'CDPRODFIN'     => 'P_CDPRODFINA',
                    'IDDIFERENCA'   => 'P_IDDIFERENCA',
                    'DIFERPERC'     => 'P_PEDIFERENCA',
                    'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                    'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                ),
            ),
        'cus00804' => 
            array (
                'strategy' => 'QR2',
                'parameterMapping' => 
                array (
                    'DTPERIINIC'    => 'P_DTPERIINIC',
                    'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                    'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                    'CDFILIAL'      => 'P_CDFILIAL',
                    'LISTASERV'     => 'P_CDSERVFILI',
                    'DTINICIAL'     => 'P_DTINICIAL',
                    'DTFINAL'       => 'P_DTFINAL',
                    'DIFERPERC'     => 'P_PEDIFERENCA',
                    'CDPRODINI'     => 'P_CDPRODINIC',
                    'CDPRODFIN'     => 'P_CDPRODFINA',
                    'DTPERIFINA'    => 'P_DTPERIFINA',
                    'IDDIFERENCA'   => 'P_IDDIFERENCA',
                ),
            ),
        'cus00803' => 
              array (
                'strategy' => 'QR2',
                'parameterMapping' => 
                array (
                    'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                    'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                    'CDPRODINI'     => 'P_CDPRODINIC',
                    'CDPRODFIN'     => 'P_CDPRODFINA',
                    'CDFILIAL'      => 'P_CDFILIAL',
                    'LISTASERV'     => 'P_CDSERVFILI',
                    'DTINICIAL'     => 'P_DTINICIAL',
                    'DTFINAL'       => 'P_DTFINAL',
                    'DTPERIINIC'    => 'P_DTPERIINIC',
                    'DTPERIFINA'    => 'P_DTPERIFINA',
                    'IDDIFERENCA'   => 'P_IDDIFERENCA',
                    'DIFERPERC'     => 'P_PEDIFERENCA',
                ),
            ),
        'cus00802' => 
            array (
                'strategy' => 'QR2',
                'parameterMapping' => 
                array (
                    'DIFERPERC'     => 'P_PEDIFERENCA',
                    'DTPERIINIC'    => 'P_DTPERIINIC',
                    'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                    'CDPRODINI'     => 'P_CDPRODINIC',
                    'CDPRODFIN'     => 'P_CDPRODFINA',
                    'CDFILIAL'      => 'P_CDFILIAL',
                    'DTINICIAL'     => 'P_DTINICIAL',
                    'DTFINAL'       => 'P_DTFINAL',
                    'IDDIFERENCA'   => 'P_IDDIFERENCA',
                    'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                    'LISTASERV'     => 'P_CDSERVFILI',
                    'DTPERIFINA'    => 'P_DTPERIFINA',
                ),
            ),    
        'cus00800' => 
            array (
                'strategy' => 'QR2',
                'parameterMapping' => 
                array (
                    'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                    'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                    'LISTASERV'     => 'P_CDSERVFILI',
                    'DTINICIAL'     => 'P_DTINICIAL',
                    'CDPRODINI'     => 'P_CDPRODINIC',
                    'DTPERIINIC'    => 'P_DTPERIINIC',
                    'DTPERIFINA'    => 'P_DTPERIFINA',
                    'NIVEL'         => 'P_NIVEL',
                    'CDPRODFIN'     => 'P_CDPRODFINA',
                    'ORDENA'        => 'P_ORDENA',
                    'IDDIFERENCA'   => 'P_IDDIFERENCA',
                    'DIFERPERC'     => 'P_PEDIFERENCA',
                    'DTFINAL'       => 'P_DTFINAL',
                    'NRCONSPLAN'    => 'P_NRCONSPLAN',
                    'CDFILIAL'      => 'P_CDFILIAL',
                ),
            ),
        'cus00801' => 
        array (
            'strategy' => 'QR2',
            'parameterMapping' => 
            array (
                'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                'CDFILIAL'      => 'P_CDFILIAL',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'DTPERIINIC'    => 'P_DTPERIINIC',
                'DTPERIFINA'    => 'P_DTPERIFINA',
                'IDDIFERENCA'   => 'P_IDDIFERENCA',
                'DIFERPERC'     => 'P_PEDIFERENCA',
                'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                'LISTASERV'     => 'P_CDSERVFILI',
                'DTINICIAL'     => 'P_DTINICIAL',
                'DTFINAL'       => 'P_DTFINAL',
                'CDPRODINI'     => 'P_CDPRODINIC',
                'CDPRODFIN'     => 'P_CDPRODFINA',
            ),
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $dataFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dataInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);

        $tamanho = 10;
        $completaCom = 'z';
        $row['CDPRODFIN'] = str_pad($row['CDPRODFIN'], $tamanho, $completaCom, STR_PAD_RIGHT);
        
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);

        $nrorg      = $row['NRORG'];
        $cdfilial   = $row['CDFILIAL'];
        $cdoperador = $row['CDOPERADOR'];
        $cdgrupserv = $row ['LISTAGRUPSERV'];
        if ($row['LISTASERV'] == 'REQUEST') {
            $row['LISTASERV'] = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
        }
        
        if($row['NIVEL'] == 0) {
            $row['NIVEL'] = 'N';
        } else {
            $row['NIVEL'] = 'S';
        }

        if($row["ORDENA"] === "1") {
            $row["NRTAMCODIGO"] = 0;
        } else {
            //numero de caracteres = tamanho total da máscara - os pontos separadores
            $row["NRTAMCODIGO"] = strlen($row["PROD_MASK"]) - substr_count($row["PROD_MASK"], ".");
        }

        $nrConsPlan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$cdfilial, $nrorg);

        $this->procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],  Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $row['LISTASERV'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],   Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",               Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",             Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "N",               Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                 Param::PARAM_TYPE_INT));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",               Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrConsPlan,       Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $row['CDPRODINI'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $row['CDPRODFIN'],  Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],    Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dataInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dataFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));

        $resultado = $this->procedureConsumoPlanejado->execute();

        $row['NRCONSPLAN'] = $nrConsPlan;
        $row['DTPERIINIC'] = $row['DTINICIAL'];
        $row['DTPERIFINA'] = $row['DTFINAL'];
    }

    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        
        $requestRow=$request->getRow();

        $parametros = array("NRORG" => $requestRow["NRORG"], "CDFILICPLA" => $requestRow["CDFILIAL"],
                                "NRCONSPLAN" => $requestRow["NRCONSPLAN"], "__is_new" => false);
        
        $newRequest = new DTO\Request\Row($parametros, $request->getMethod(),
                                            $request->getRoutePath(), $request->getUserId());
        $dummyResponse = new DTO\Response();
        $this->consplan->delete($newRequest, $dummyResponse);
    }  
}